/***************************************************************************
 *   Copyright (C) 2007 Mikhail Fedotov                                    *
 *   anyremote@mail.ru                                                     *
 *   http://anyremote.sf.net                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* ANYREMOTE PLUGIN FOR THE KOPETE KDE INSTANT MESSANGER 
 *
 * base class implementation
 *
 */
#include <iostream.h>
#include <qfile.h>
#include <qdir.h>

#include <qregexp.h>
#include <qtimer.h>
#include <kdebug.h>
#include <kapplication.h>
//#include <dcopclient.h>
#include <kgenericfactory.h>
#include <kaction.h>
#include <kaboutdata.h>
#include <kmessagebox.h>

#include <kopetemetacontact.h>
#include <kopetecontactlist.h>
#include <kopeteview.h>
#include <kopeteaccount.h>
#include <kopeteaccountmanager.h>
#include <kopetechatsessionmanager.h>

#include "kopeteanyremoteuserprefs.h"
#include "kopete_anyremote.h"
#include "kopeteanyremoteplugin.h"

typedef KGenericFactory<KopeteAnyremotePlugin> KopeteAnyremotePluginFactory;

static const KAboutData aboutdata( "kopete_anyremote", I18N_NOOP( "KopeteAnyremote" ) , "1.3.0" );
K_EXPORT_COMPONENT_FACTORY( kopete_anyremote, KopeteAnyremotePluginFactory( &aboutdata ) )


KopeteAnyremotePlugin::KopeteAnyremotePlugin( QObject *parent, const char *name, const QStringList & )
		: /*DCOPStub( "anyremoted", "KSpeech" ),*/
		Kopete::Plugin( KopeteAnyremotePluginFactory::instance(), parent, name )
		
{
	kdDebug() << k_funcinfo << "Starting ANYREMOTE Plugin" << endl;
	
	if ( ! pluginStatic_ )
	{
		pluginStatic_ = this;
	}

	// load configuration
	slotSettingsChanged();

	// connect the statusChanged slots
	initialize();
	
	// add item to (contact) menu
	KAction *showUserPrefsDialog = new KAction( i18n( "&Text-to-Speech settings" ), QString::fromLatin1( "kmouth" ), 0, this, SLOT( slotShowUserPrefsDialog() ), actionCollection() , "showUserPrefsDialog" );
	QObject::connect ( Kopete::ContactList::self() , SIGNAL( metaContactSelected( bool ) ) , showUserPrefsDialog , SLOT( setEnabled( bool ) ) );
	showUserPrefsDialog->setEnabled( Kopete::ContactList::self() ->selectedMetaContacts().count() == 1 );
	setXMLFile( "anyremoteui.rc" );

	// look out for new meta contacts
	QObject::connect( Kopete::ContactList::self(), SIGNAL( metaContactAdded( Kopete::MetaContact* ) ), this, SLOT( slotMetaContactAdded( Kopete::MetaContact* ) ) );
	// listen for incoming messages
	QObject::connect( Kopete::ChatSessionManager::self(), SIGNAL( aboutToReceive( Kopete::Message& ) ), this, SLOT( slotIncomingMessage( Kopete::Message& ) ) );
	// listen for outgoing messages
	QObject::connect( Kopete::ChatSessionManager::self(), SIGNAL( aboutToSend( Kopete::Message& ) ), this, SLOT( slotOutgoingMessage( Kopete::Message& ) ) );
	// listen for config changes
	QObject::connect( this, SIGNAL( settingsChanged() ), this, SLOT( slotSettingsChanged() ) );
	// track newly created chat sessions
	QObject::connect( Kopete::ChatSessionManager::self(), SIGNAL( chatSessionCreated(Kopete::ChatSession*) ), this, SLOT( slotSessionCreated( Kopete::ChatSession* ) ) );

	QObject::connect( Kopete::AccountManager::self(), SIGNAL( accountOnlineStatusChanged(Kopete::Account *, const Kopete::OnlineStatus &, const Kopete::OnlineStatus &)), this, SLOT(slotAccountOnlineStatusChanged()));

	// see if anyremoted is running, if not try to start it...
	//DCOPClient *client = KApplication::dcopClient();
	//if ( ! client->isAttached() )
	//{
	//	client->attach();
	//}
	
	
}

KopeteAnyremotePlugin::~KopeteAnyremotePlugin()
{
	pluginStatic_ = 0L;

	QDir dir("/");
	dir.remove("/tmp/kopete-anyremote.status");
}


// type:
//	0 - status change
//      1 - incoming message
//      2 - outgoing message
//
void KopeteAnyremotePlugin::dumpMessage(QString id, QString data, int type) {
	cout << "DUMP " << id << " " << type << endl;

	QString id2 = id.section(":",2,2);
	
	QString fName("/tmp/kopete-anyremote.status");
	if (type == 1 || type == 2) {
		fName = "/tmp/kopete-anyremote.msg."+id2;
	}
	QString fDup = fName + ".dup";

	QFile ff(fName);
	QDir dir("/");
	if (ff.exists()) {
		if (!dir.rename(fName, fDup)) {
			cout << "ERROR: can't create "<< fDup << endl;
		}
	} 
	
	QFile fdup(fDup);
	if (fdup.open(IO_WriteOnly|IO_Append)) {
		QTextStream stream(&fdup);
		if (type == 0) {
			stream << id2 << ";";
		}
		stream << data << endl;
		fdup.close();

		if (!dir.rename(fDup, fName)) {
			cout << "ERROR: can't create " << fName << endl;
			return;
		}
	} else {
		cout << "ERROR: can't open " << fDup << endl;
		return;
	}
}

void KopeteAnyremotePlugin::delFile(QString file) {
	QFile ff(file);
	if (ff.exists()) {
		QFile::remove(file);
	} 
}

void KopeteAnyremotePlugin::dumpAllContactsInfo(QString mcId) {
	cout << "dumpAllContactsInfo" << endl;
	
	QPtrList<Kopete::MetaContact> metalist = Kopete::ContactList::self()->metaContacts();
	QPtrList<Kopete::Contact> list;
	Kopete::MetaContact* mc;
	
	delFile("/tmp/kopete-anyremote.status");
	
	for ( mc = metalist.first(); mc != 0; mc = metalist.next() )
	{
		// dump current status
		KopeteAnyremoteUserPrefs* prefs = getUserPrefs(mc);
		Kopete::OnlineStatus::StatusType st = mc->status();
		QString data = prefs->getName() + ";";
		if (mcId != QString::null && mcId == mc->metaContactId()) {	// in/out message 
			data += "Msg;"+Kopete::OnlineStatus::statusTypeToString(st);
		} else {
			data += Kopete::OnlineStatus::statusTypeToString(st);
		}
		dumpMessage(mc->metaContactId(),data,0);
	}
}

KopeteAnyremotePlugin* KopeteAnyremotePlugin::pluginStatic_ = 0L;
;

KopeteAnyremotePlugin* KopeteAnyremotePlugin::self()
{
	return pluginStatic_ ;
}


void KopeteAnyremotePlugin::initialize()
{
	
	delFile("/tmp/kopete-anyremote.status");

	QPtrList<Kopete::MetaContact> metalist = Kopete::ContactList::self()->metaContacts();
	QPtrList<Kopete::Contact> list;
	Kopete::MetaContact* mc;
	for ( mc = metalist.first(); mc != 0; mc = metalist.next() )
	{
		QObject::connect( mc, SIGNAL( onlineStatusChanged( Kopete::MetaContact*, Kopete::OnlineStatus::StatusType ) ), this, SLOT( slotOnlineStatusChanged( Kopete::MetaContact*, Kopete::OnlineStatus::StatusType ) ) );
	}
	dumpAllContactsInfo(QString::null);
}


KopeteAnyremoteUserPrefs* KopeteAnyremotePlugin::getUserPrefs( Kopete::MetaContact* mc )
{
	// return user specific preferences. create them if necessary
	KopeteAnyremoteUserPrefs* prefs = this->userPrefs.find( mc->metaContactId() );
	if ( prefs == 0 )
	{
		prefs = new KopeteAnyremoteUserPrefs( mc );
		this->userPrefs.insert( mc->metaContactId(), prefs );
	}
	return prefs;
}


//
// SLOTS
//

void KopeteAnyremotePlugin::slotSettingsChanged()
{
	kdDebug() << k_funcinfo << "slotSettingsChanged() triggered" << endl;
	KopeteAnyremoteKcfg::self()->readConfig();
}


void KopeteAnyremotePlugin::slotSessionCreated( Kopete::ChatSession* session )
{
	freshSessions.append( session );
}


void KopeteAnyremotePlugin::slotIncomingMessage( Kopete::Message& msg )
{
	Kopete::MetaContact *from = msg.from()->metaContact();
	KopeteAnyremoteUserPrefs* prefs = getUserPrefs(from);
	
	if (prefs->getReadIncomingMessages()) {

		QString name = prefs->getName();
		
		dumpAllContactsInfo(from->metaContactId());
		dumpMessage(from->metaContactId(),"From "+name+": "+msg.plainBody(),1);
	}
}


void KopeteAnyremotePlugin::slotOutgoingMessage( Kopete::Message& msg )
{
	// Sad, we have to consider only the first To: metacontact
	Kopete::MetaContact *to = msg.to().first()->metaContact();
	KopeteAnyremoteUserPrefs* prefs = getUserPrefs(to);
	
	if (prefs->getReadOutgoingMessages()) {
		
		QString name = prefs->getName();
		
		dumpAllContactsInfo(to->metaContactId());
		dumpMessage(to->metaContactId(),"To "+name+": "+msg.plainBody(),2);
	}
}


void KopeteAnyremotePlugin::slotMetaContactAdded( Kopete::MetaContact* mc )
{
	QObject::connect( mc, SIGNAL( onlineStatusChanged( Kopete::MetaContact*, Kopete::OnlineStatus::StatusType ) ), this, SLOT( slotOnlineStatusChanged( Kopete::MetaContact*, Kopete::OnlineStatus::StatusType ) ) );
}


void KopeteAnyremotePlugin::slotOnlineStatusChanged( Kopete::MetaContact* mc, Kopete::OnlineStatus::StatusType status )
{
	cout << "slotOnlineStatusChanged" << endl;
	
	// disable status announcements while any account has the suppressStatusNotification flag set
	/*QPtrList<Kopete::Account> accounts = Kopete::AccountManager::self()->accounts();
	Kopete::Account* a;
	for (a = accounts.first(); a != 0; a = accounts.next()) {
		if (a->suppressStatusNotification()) {
			return;
		}
	}
	*/
	dumpAllContactsInfo(QString::null);
	
	QTimer::singleShot(7000, this, SLOT(slotTimerDump()));
}

void KopeteAnyremotePlugin::slotTimerDump()
{
	cout << "slotTimerDump" << endl;
	dumpAllContactsInfo(QString::null);
}

void KopeteAnyremotePlugin::slotAccountOnlineStatusChanged()
{
	cout << "slotAccountOnlineStatusChanged" << endl;
	dumpAllContactsInfo(QString::null);
}

void KopeteAnyremotePlugin::slotShowUserPrefsDialog()
{
	// get the clicked contact
	Kopete::MetaContact* mc = Kopete::ContactList::self()->selectedMetaContacts().first();
	if ( mc )
	{
		// show his settings
		KopeteAnyremoteUserPrefs* userPrefs = getUserPrefs( mc );
		userPrefs->showContactSettingsDialog( );
	}
}

/*void KopeteAnyremotePlugin::announceStatusChange() // KopeteAnyremoteUserPrefs* prefs, QString mcId, QString statusDescription )
{
	cout << "announceStatusChange" << endl;
	
 	//QString text = mcId + ";" + prefs->getName() + ";" + statusDescription;
	//dumpMessage(text, QString::null, 0);
	dumpAllContactsInfo(QString::null);
}
*/
#include "kopeteanyremoteplugin.moc"
